==> org-lsst-ccs-subsystem-ocsbridge-main-2.8.0-sources.jar: org/lsst/ccs/subsystem/ocsbridge/util/SummaryInfoConverter.java <==
package org.lsst.ccs.subsystem.ocsbridge.util;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.Collector;
import org.lsst.ccs.bus.data.ConfigurationInfo;
import org.lsst.ccs.bus.data.ConfigurationParameterInfo;
import org.lsst.ccs.subsystem.focalplane.data.SequencerType;
import org.lsst.ccs.subsystem.ocsbridge.config.Device;
import org.lsst.sal.camera.CameraEvent;
import org.lsst.sal.camera.event.FocalPlaneSummaryInfoEvent.CcdType;

/**
 * Generates a special event built for the header service from a focal-plane
 * configuration info object.
 *
 * @author tonyj
 */
public class SummaryInfoConverter {

    private final static Pattern SENSOR_PATTERN = Pattern.compile("(R..)/(Reb.)/(S..)/(\\w+)");
    private final static Pattern REB_PATTERN = Pattern.compile("(R..)/(Reb.)(?:_hardware)?/(\\w+)");
    private final static Pattern RAFT_PATTERN = Pattern.compile("(R..)/(.+)");
    private final static DelimitedStringSplitJoin DELIMITED_STRING_SPLIT_JOIN = new DelimitedStringSplitJoin();
    private final Device device;

    public SummaryInfoConverter(Device device) {
        this.device = device;
    }
    
    public CameraEvent convert(ConfigurationInfo data) {
        Set<String> ccdLocations = new TreeSet<>();
        Map<String, String> ccdNames = new HashMap<>();
        Map<String, String> manSerNums = new HashMap<>();
        Map<String, String> rebNames = new HashMap<>();
        Map<String, String> rebSerialNums = new HashMap<>();
        Map<String, String> raftNames = new HashMap<>();
        Map<String, String> ccdNameToRebName = new HashMap<>();
        Map<String, Double> setTemp = new HashMap<>();
        Map<SequencerType, String> seqMap = new HashMap<>();
        Map<SequencerType, Long> seqChecksumMap = new HashMap<>(); 

        // First loop over hardware Ids
        Map<String, String> hardwareMap = data.getCurrentValuesForCategory("HardwareId");
        hardwareMap.forEach((key, value) -> {
            //System.out.println(entry.getKey() + "=" + entry.getValue());
            Matcher sensorMatcher = SENSOR_PATTERN.matcher(key);
            Matcher rebMatcher = REB_PATTERN.matcher(key);
            Matcher raftMatcher = RAFT_PATTERN.matcher(key);

            if (sensorMatcher.matches()) {
                final String ccdName = sensorMatcher.group(1) + sensorMatcher.group(3);
                final String rebName = sensorMatcher.group(1) + sensorMatcher.group(2);
                ccdLocations.add(ccdName);
                ccdNameToRebName.put(ccdName, rebName);
                String type = sensorMatcher.group(4);
                if ("name".equals(type)) {
                    ccdNames.put(ccdName, value);
                } else if ("manSerNum".equals(type)) {
                    manSerNums.put(ccdName, value);
                }
            } else if (rebMatcher.matches()) {
                final String rebName = rebMatcher.group(1) + rebMatcher.group(2);
                String type = rebMatcher.group(3);
                if ("name".equals(type)) {
                    rebNames.put(rebName, value);
                }
            } else if (raftMatcher.matches()) {
                final String raftName = raftMatcher.group(1);
                String type = raftMatcher.group(2);
                if ("name".equals(type)) {
                    raftNames.put(raftName, value);
                }
            }
        });
        Map<String, String> raftsMap = data.getCurrentValuesForCategory("Rafts");
        raftsMap.forEach((key, value) -> {
            //System.out.println(entry.getKey() + "=" + entry.getValue());
            Matcher rebMatcher = REB_PATTERN.matcher(key);
            if (rebMatcher.matches()) {
                final String rebName = rebMatcher.group(1) + rebMatcher.group(2);
                String type = rebMatcher.group(3);
                if ("serialNum".equals(type)) {
                    rebSerialNums.put(rebName, value);
                }
            }
        });

        Map<String, String> tempControlMap = data.getCurrentValuesForCategory("RaftTempControl");
        tempControlMap.forEach((key, value) -> {
            Matcher raftMatcher = RAFT_PATTERN.matcher(key);
            if (raftMatcher.matches()) {
                final String raftName = raftMatcher.group(1);
                String type = raftMatcher.group(2);
                if ("TempControl/setTemp".equals(type)) {
                    setTemp.put(raftName, Double.parseDouble(value));
                }
            }
        });
        
        List<ConfigurationParameterInfo> allParameterInfo = data.getAllParameterInfo();
        allParameterInfo.forEach(pi -> {
            if ("sequencerConfig/sequencer".equals(pi.getPathName())) {
                seqMap.putAll((Map<SequencerType, String>) pi.getConfiguredValueObject());
            } else if ("sequencerConfig/sequencerChecksums".equals(pi.getPathName())) {          
                seqChecksumMap.putAll(((Map<SequencerType, Long>) pi.getConfiguredValueObject()));
            }
        });
        // FIXME: Because auxtel FocalPlaneSummaryInfoEvent has dimension 1 it is not being mapped onto camera FocalPlaneSummaryInfoEvent 
        if (device == Device.AUXTEL) {
            if (ccdLocations.size() != 1) {
                throw new RuntimeException("Incorrect length for AuxTel locations");
            }
            String theCcdLocation = ccdLocations.iterator().next();
            return org.lsst.sal.atcamera.event.FocalPlaneSummaryInfoEvent.builder()
                    .ccdLocation(ccdLocations.stream().collect(join()))
                    .raftBay(ccdLocations.stream().map(SummaryInfoConverter::raftBayForCCDLocation).collect(join()))
                    .ccdSlot(ccdLocations.stream().map(SummaryInfoConverter::ccdSlotForCCDLocation).collect(join()))
                    .raftLSSTName(ccdLocations.stream().map(SummaryInfoConverter::raftBayForCCDLocation).map(name -> raftNames.get(name)).collect(join()))
                    .ccdLSSTName(ccdLocations.stream().map(k -> ccdNames.get(k)).collect(join()))
                    .ccdManufacturer(ccdLocations.stream().map(k -> ccdNames.get(k).substring(0, 3)).collect(join()))
                    .ccdType(org.lsst.sal.atcamera.event.FocalPlaneSummaryInfoEvent.CcdType.ITL)
                    .ccdManSerNum(ccdLocations.stream().map(k -> get(manSerNums, k)).collect(join()))
                    .rebSerialNumber(ccdLocations.stream().map(ccd -> rebSlotForCCDLocation(ccdNameToRebName, ccd)).map(name -> rebSerialNums.get(name)).collect(join()))
                    .rebLSSTName(ccdLocations.stream().map(ccd -> rebSlotForCCDLocation(ccdNameToRebName, ccd)).map(name -> get(rebNames, name)).collect(join()))
                    .ccdTempSetPoint(get(setTemp, theCcdLocation, -999.0))
                    .sequencerKey(ccdLocations.stream().map(k -> ccdNames.get(k).substring(0, 3)).map(k -> SequencerType.valueOf(k)).map(k -> get(seqMap, k)).collect(join()))
                    .sequencerChecksum(ccdLocations.stream().map(k -> ccdNames.get(k).substring(0, 3)).map(k -> SequencerType.valueOf(k)).map(k -> get(seqChecksumMap, k, 0l)).map(l -> String.valueOf(l)).collect(join()))
                    .build();            
        } else {
            return org.lsst.sal.camera.event.FocalPlaneSummaryInfoEvent.builder()
                    .ccdLocation(ccdLocations.stream().collect(join()))
                    .raftBay(ccdLocations.stream().map(SummaryInfoConverter::raftBayForCCDLocation).collect(join()))
                    .ccdSlot(ccdLocations.stream().map(SummaryInfoConverter::ccdSlotForCCDLocation).collect(join()))
                    .raftLSSTName(ccdLocations.stream().map(SummaryInfoConverter::raftBayForCCDLocation).map(name -> raftNames.get(name)).collect(join()))
                    .ccdLSSTName(ccdLocations.stream().map(k -> ccdNames.get(k)).collect(join()))
                    .ccdManufacturer(ccdLocations.stream().map(k -> ccdNames.get(k).substring(0, 3)).collect(join()))
                    .ccdType(ccdLocations.stream().map(k -> ccdNames.get(k).substring(0, 3)).map(k -> CcdType.valueOf(k)).toArray(CcdType[]::new))
                    .ccdManSerNum(ccdLocations.stream().map(k -> get(manSerNums, k)).collect(join()))
                    .rebSerialNumber(ccdLocations.stream().map(ccd -> rebSlotForCCDLocation(ccdNameToRebName, ccd)).map(name -> rebSerialNums.get(name)).collect(join()))
                    .rebLSSTName(ccdLocations.stream().map(ccd -> rebSlotForCCDLocation(ccdNameToRebName, ccd)).map(name -> get(rebNames, name)).collect(join()))
                    .ccdTempSetPoint(ccdLocations.stream().map(SummaryInfoConverter::raftBayForCCDLocation).map(name -> get(setTemp, name, -999.0)).mapToDouble(d -> d).toArray())
                    .sequencerKey(ccdLocations.stream().map(k -> ccdNames.get(k).substring(0, 3)).map(k -> SequencerType.valueOf(k)).map(k -> get(seqMap, k)).collect(join()))
                    .sequencerChecksum(ccdLocations.stream().map(k -> ccdNames.get(k).substring(0, 3)).map(k -> SequencerType.valueOf(k)).map(k -> get(seqChecksumMap, k, 0l)).map(l -> String.valueOf(l)).collect(join()))
                    .build();
        }
    }
    private static String raftBayForCCDLocation(String ccdLocation) {
        return ccdLocation.substring(0, 3);
    }
    
    private static String ccdSlotForCCDLocation(String ccdLocation) {
        return ccdLocation.substring(3, 6);
    }
    
    private static String rebSlotForCCDLocation(Map<String, String> ccdNameToRebName, String ccdLocation) {
        return ccdNameToRebName.get(ccdLocation);
    }
    
    private static <K> String get(Map<K, String> map, K name) {
        return get(map, name, "Unknown");
    }
    
    private static <K, V> V get(Map<K, V> map, K key, V defaultValue) {
        return map.getOrDefault(key, defaultValue);
    }
    
    private static Collector<CharSequence, ?, String> join() {
        return DELIMITED_STRING_SPLIT_JOIN.joining();
    }
}

==> org-lsst-ccs-subsystem-ocsbridge-main-3.1.1-sources.jar: org/lsst/ccs/subsystem/ocsbridge/util/SummaryInfoConverter.java <==
package org.lsst.ccs.subsystem.ocsbridge.util;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.Collector;
import org.lsst.ccs.bus.data.ConfigurationInfo;
import org.lsst.ccs.bus.data.ConfigurationParameterInfo;
import org.lsst.ccs.subsystem.focalplane.data.SequencerType;
import org.lsst.ccs.subsystem.ocsbridge.config.Camera;
import org.lsst.sal.camera.CameraEvent;
import org.lsst.sal.camera.event.FocalPlaneSummaryInfoEvent.CcdType;

/**
 * Generates a special event built for the header service from a focal-plane
 * configuration info object.
 *
 * @author tonyj
 */
public class SummaryInfoConverter {

    private final static Pattern SENSOR_PATTERN = Pattern.compile("(R..)/(Reb.)/(S..)/(\\w+)");
    private final static Pattern REB_PATTERN = Pattern.compile("(R..)/(Reb.)(?:_hardware)?/(\\w+)");
    private final static Pattern RAFT_PATTERN = Pattern.compile("(R..)/(.+)");
    private final static DelimitedStringSplitJoin DELIMITED_STRING_SPLIT_JOIN = new DelimitedStringSplitJoin();
    private final Camera device;

    public SummaryInfoConverter(Camera device) {
        this.device = device;
    }
    
    public CameraEvent convert(ConfigurationInfo data) {
        Set<String> ccdLocations = new TreeSet<>();
        Map<String, String> ccdNames = new HashMap<>();
        Map<String, String> manSerNums = new HashMap<>();
        Map<String, String> rebNames = new HashMap<>();
        Map<String, String> rebSerialNums = new HashMap<>();
        Map<String, String> raftNames = new HashMap<>();
        Map<String, String> ccdNameToRebName = new HashMap<>();
        Map<String, Double> setTemp = new HashMap<>();
        Map<SequencerType, String> seqMap = new HashMap<>();
        Map<SequencerType, Long> seqChecksumMap = new HashMap<>(); 

        // First loop over hardware Ids
        Map<String, String> hardwareMap = data.getCurrentValuesForCategory("HardwareId");
        hardwareMap.forEach((key, value) -> {
            //System.out.println(entry.getKey() + "=" + entry.getValue());
            Matcher sensorMatcher = SENSOR_PATTERN.matcher(key);
            Matcher rebMatcher = REB_PATTERN.matcher(key);
            Matcher raftMatcher = RAFT_PATTERN.matcher(key);

            if (sensorMatcher.matches()) {
                final String ccdName = sensorMatcher.group(1) + sensorMatcher.group(3);
                final String rebName = sensorMatcher.group(1) + sensorMatcher.group(2);
                ccdLocations.add(ccdName);
                ccdNameToRebName.put(ccdName, rebName);
                String type = sensorMatcher.group(4);
                if ("name".equals(type)) {
                    ccdNames.put(ccdName, value);
                } else if ("manSerNum".equals(type)) {
                    manSerNums.put(ccdName, value);
                }
            } else if (rebMatcher.matches()) {
                final String rebName = rebMatcher.group(1) + rebMatcher.group(2);
                String type = rebMatcher.group(3);
                if ("name".equals(type)) {
                    rebNames.put(rebName, value);
                }
            } else if (raftMatcher.matches()) {
                final String raftName = raftMatcher.group(1);
                String type = raftMatcher.group(2);
                if ("name".equals(type)) {
                    raftNames.put(raftName, value);
                }
            }
        });
        Map<String, String> raftsMap = data.getCurrentValuesForCategory("Rafts");
        raftsMap.forEach((key, value) -> {
            //System.out.println(entry.getKey() + "=" + entry.getValue());
            Matcher rebMatcher = REB_PATTERN.matcher(key);
            if (rebMatcher.matches()) {
                final String rebName = rebMatcher.group(1) + rebMatcher.group(2);
                String type = rebMatcher.group(3);
                if ("serialNum".equals(type)) {
                    rebSerialNums.put(rebName, value);
                }
            }
        });

        Map<String, String> tempControlMap = data.getCurrentValuesForCategory("RaftTempControl");
        tempControlMap.forEach((key, value) -> {
            Matcher raftMatcher = RAFT_PATTERN.matcher(key);
            if (raftMatcher.matches()) {
                final String raftName = raftMatcher.group(1);
                String type = raftMatcher.group(2);
                if ("TempControl/setTemp".equals(type)) {
                    setTemp.put(raftName, Double.parseDouble(value));
                }
            }
        });
        
        List<ConfigurationParameterInfo> allParameterInfo = data.getAllParameterInfo();
        allParameterInfo.forEach(pi -> {
            if ("sequencerConfig/sequencer".equals(pi.getPathName())) {
                seqMap.putAll((Map<SequencerType, String>) pi.getCurrentValueObject());
            } else if ("sequencerConfig/sequencerChecksums".equals(pi.getPathName())) {          
                seqChecksumMap.putAll(((Map<SequencerType, Long>) pi.getCurrentValueObject()));
            }
        });
        // FIXME: Because auxtel FocalPlaneSummaryInfoEvent has dimension 1 it is not being mapped onto camera FocalPlaneSummaryInfoEvent 
        if (device == Camera.AUXTEL) {
            if (ccdLocations.size() != 1) {
                throw new RuntimeException("Incorrect length for AuxTel locations");
            }
            String theCcdLocation = ccdLocations.iterator().next();
            return org.lsst.sal.atcamera.event.FocalPlaneSummaryInfoEvent.builder()
                    .ccdLocation(ccdLocations.stream().collect(join()))
                    .raftBay(ccdLocations.stream().map(SummaryInfoConverter::raftBayForCCDLocation).collect(join()))
                    .ccdSlot(ccdLocations.stream().map(SummaryInfoConverter::ccdSlotForCCDLocation).collect(join()))
                    .raftLSSTName(ccdLocations.stream().map(SummaryInfoConverter::raftBayForCCDLocation).map(name -> raftNames.get(name)).collect(join()))
                    .ccdLSSTName(ccdLocations.stream().map(k -> ccdNames.get(k)).collect(join()))
                    .ccdManufacturer(ccdLocations.stream().map(k -> ccdNames.get(k).substring(0, 3)).collect(join()))
                    .ccdType(org.lsst.sal.atcamera.event.FocalPlaneSummaryInfoEvent.CcdType.ITL)
                    .ccdManSerNum(ccdLocations.stream().map(k -> get(manSerNums, k)).collect(join()))
                    .rebSerialNumber(ccdLocations.stream().map(ccd -> rebSlotForCCDLocation(ccdNameToRebName, ccd)).map(name -> rebSerialNums.get(name)).collect(join()))
                    .rebLSSTName(ccdLocations.stream().map(ccd -> rebSlotForCCDLocation(ccdNameToRebName, ccd)).map(name -> get(rebNames, name)).collect(join()))
                    .ccdTempSetPoint(get(setTemp, theCcdLocation, -999.0))
                    .sequencerKey(ccdLocations.stream().map(k -> ccdNames.get(k).substring(0, 3)).map(k -> SequencerType.valueOf(k)).map(k -> get(seqMap, k)).collect(join()))
                    .sequencerChecksum(ccdLocations.stream().map(k -> ccdNames.get(k).substring(0, 3)).map(k -> SequencerType.valueOf(k)).map(k -> get(seqChecksumMap, k, 0l)).map(l -> String.valueOf(l)).collect(join()))
                    .build();            
        } else {
            return org.lsst.sal.camera.event.FocalPlaneSummaryInfoEvent.builder()
                    .ccdLocation(ccdLocations.stream().collect(join()))
                    .raftBay(ccdLocations.stream().map(SummaryInfoConverter::raftBayForCCDLocation).collect(join()))
                    .ccdSlot(ccdLocations.stream().map(SummaryInfoConverter::ccdSlotForCCDLocation).collect(join()))
                    .raftLSSTName(ccdLocations.stream().map(SummaryInfoConverter::raftBayForCCDLocation).map(name -> raftNames.get(name)).collect(join()))
                    .ccdLSSTName(ccdLocations.stream().map(k -> ccdNames.get(k)).collect(join()))
                    .ccdManufacturer(ccdLocations.stream().map(k -> ccdNames.get(k).substring(0, 3)).collect(join()))
                    .ccdType(ccdLocations.stream().map(k -> ccdNames.get(k).substring(0, 3)).map(k -> CcdType.valueOf(k)).toArray(CcdType[]::new))
                    .ccdManSerNum(ccdLocations.stream().map(k -> get(manSerNums, k)).collect(join()))
                    .rebSerialNumber(ccdLocations.stream().map(ccd -> rebSlotForCCDLocation(ccdNameToRebName, ccd)).map(name -> rebSerialNums.get(name)).collect(join()))
                    .rebLSSTName(ccdLocations.stream().map(ccd -> rebSlotForCCDLocation(ccdNameToRebName, ccd)).map(name -> get(rebNames, name)).collect(join()))
                    .ccdTempSetPoint(ccdLocations.stream().map(SummaryInfoConverter::raftBayForCCDLocation).map(name -> get(setTemp, name, -999.0)).mapToDouble(d -> d).toArray())
                    .sequencerKey(ccdLocations.stream().map(k -> ccdNames.get(k).substring(0, 3)).map(k -> SequencerType.valueOf(k)).map(k -> get(seqMap, k)).collect(join()))
                    .sequencerChecksum(ccdLocations.stream().map(k -> ccdNames.get(k).substring(0, 3)).map(k -> SequencerType.valueOf(k)).map(k -> get(seqChecksumMap, k, 0l)).map(l -> String.valueOf(l)).collect(join()))
                    .build();
        }
    }
    private static String raftBayForCCDLocation(String ccdLocation) {
        return ccdLocation.substring(0, 3);
    }
    
    private static String ccdSlotForCCDLocation(String ccdLocation) {
        return ccdLocation.substring(3, 6);
    }
    
    private static String rebSlotForCCDLocation(Map<String, String> ccdNameToRebName, String ccdLocation) {
        return ccdNameToRebName.get(ccdLocation);
    }
    
    private static <K> String get(Map<K, String> map, K name) {
        return get(map, name, "Unknown");
    }
    
    private static <K, V> V get(Map<K, V> map, K key, V defaultValue) {
        return map.getOrDefault(key, defaultValue);
    }
    
    private static Collector<CharSequence, ?, String> join() {
        return DELIMITED_STRING_SPLIT_JOIN.joining();
    }
}
